@@ -98,13 +98,13 @@ CmdBase::~CmdBase()
 
 int CmdBase::parser(char *p)
 {
-	if (p != nullptr)
-		m_cmd = p;
-
 	size_t pos = 0;
-	string param = get_next_param(m_cmd, pos);
+	string param;
 
-	if (param.find(':') != string::npos)
+	if (parser_protocol(p, pos))
+		return -1;
+
+	if (pos < m_cmd.size())
 		param = get_next_param(m_cmd, pos);
 
 	size_t index = 0;
