@@ -30,6 +30,7 @@
 */
 
 #include <string>
+#include "config.h"
 #include "sdps.h"
 #include "hidreport.h"
 #include "liberror.h"
@@ -239,6 +240,64 @@ SDPBootCmd::SDPBootCmd(char *p) : SDPCmdBase(p)
 	insert_param_info("-cleardcd", &m_clear_dcd, Param::Type::e_bool);
 	insert_param_info("-dcdaddr", &m_dcd_addr, Param::Type::e_uint32);
 	insert_param_info("-scanlimited", &m_scan_limited, Param::Type::e_uint64);
+	insert_param_info("-barebox", &m_barebox, Param::Type::e_bool);
+}
+
+# define BAREBOX_MAGIC_OFFSET	0x20
+
+bool SDPBootCmd::is_barebox_img(void)
+{
+	shared_ptr<FileBuffer> fbuf= get_file_buffer(m_filename, true);
+	if (fbuf == nullptr)
+		return false;
+
+	string barebox_magic ("barebox");
+
+	shared_ptr<DataBuffer> dbuf = fbuf->request_data(0, BAREBOX_MAGIC_OFFSET + barebox_magic.length());
+	if (dbuf == nullptr)
+		return false;
+
+	string img ((const char *)&dbuf->at(BAREBOX_MAGIC_OFFSET), barebox_magic.length());
+
+	return img.compare(barebox_magic) == 0 ? true : false;
+}
+
+int SDPBootCmd::load_barebox(CmdCtx *ctx)
+{
+	const ROM_INFO *rom = search_rom_info(ctx->m_config_item);
+
+	if (!rom)
+		return 0;
+
+	// The barebox USB loading mechanism differs between SoCs due to SRAM
+	// size limitations.
+	//
+	// E.g. all i.MX8M SoCs require a two stage loading, first the
+	// pre-bootloader (PBL) is loaded to the internal SRAM and started. The
+	// PBL intialize the DDR and uses the BootROM initialized USB
+	// controller to load the remaining "full barebox image" to the DDR.
+	//
+	// On i.MX6 devices this is not the case since the DDR setup is done
+	// via the DCD and we can load the image directly to the DDR.
+	//
+	// The ROM_INFO_NEED_BAREBOX_FULL_IMAGE flag indicates if the two-stage
+	// load mechanism is required for a specific SoC. So the user can
+	// always specify '-barebox' no matter if it's required or not:
+	//
+	// SDP: boot -barebox -f <barebox.img>
+	if (!(rom->flags & ROM_INFO_NEED_BAREBOX_FULL_IMAGE))
+		return 0;
+
+	string str;
+	str = "SDP: write -f ";
+	str += m_filename;
+	str += " -ivt 0";
+	str += " -barebox-bl33";
+
+	SDPWriteCmd wr((char *)str.c_str());
+	if (wr.parser()) return -1;
+
+	return wr.run(ctx);
 }
 
 int SDPBootCmd::run(CmdCtx *ctx)
@@ -301,6 +360,11 @@ int SDPBootCmd::run(CmdCtx *ctx)
 		if (jmp.run(ctx)) return -1;
 	}
 
+	if (m_barebox || is_barebox_img())
+	{
+		if (load_barebox(ctx)) return -1;
+	}
+
 	SDPBootlogCmd log(nullptr);
 	log.run(ctx);
 
@@ -344,11 +408,13 @@ SDPWriteCmd::SDPWriteCmd(char *p) : SDPCmdBase(p)
 	m_download_addr = 0;
 	m_bskipspl = false;
 	m_bscanterm = false;
+	m_barebox_bl33 = false;
 
 	insert_param_info("write", nullptr, Param::Type::e_null);
 	insert_param_info("-f", &m_filename, Param::Type::e_string_filename);
 	insert_param_info("-ivt", &m_Ivt, Param::Type::e_uint32);
 	insert_param_info("-addr", &m_download_addr, Param::Type::e_uint32);
+	insert_param_info("-barebox-bl33", &m_barebox_bl33, Param::Type::e_bool);
 	insert_param_info("-offset", &m_offset, Param::Type::e_uint32);
 	insert_param_info("-skipspl", &m_bskipspl, Param::Type::e_bool);
 	insert_param_info("-skipfhdr", &m_bskipfhdr, Param::Type::e_bool);
@@ -360,6 +426,7 @@ int SDPWriteCmd::run(CmdCtx*ctx)
 	size_t size;
 	uint8_t *pbuff;
 	ssize_t offset = 0;
+	bool validate_run = true;
 
 	shared_ptr<FileBuffer> p1= get_file_buffer(m_filename, true);
 
@@ -467,12 +534,26 @@ int SDPWriteCmd::run(CmdCtx*ctx)
 		if (size > fbuff->size() - off)
 			size = fbuff->size() - off;
 
+		if (m_barebox_bl33) {
+			if (pIvt->IvtBarker != IVT_BARKER_HEADER) {
+				set_last_err_string("Barebox BL33 loading is only support for IVT Header V2");
+				return -1;
+			}
+			offset = pIvt->ImageStartAddr - pIvt->SelfAddr;
+			size = fbuff->size() - off - offset;
+			m_download_addr = 0;
+
+			// Barebox does start as soon as the bl33 was loaded
+			// and the check_ack() fails
+			validate_run = false;
+		}
+
 		pbuff = (uint8_t*)pIvt;
 	}
-	return run(ctx, pbuff + offset, size, m_download_addr);
+	return run(ctx, pbuff + offset, size, m_download_addr, validate_run);
 }
 
-int SDPWriteCmd::run(CmdCtx *ctx, void *pbuff, size_t size, uint32_t addr)
+int SDPWriteCmd::run(CmdCtx *ctx, void *pbuff, size_t size, uint32_t addr, bool validate)
 {
 	HIDTrans dev(m_timeout);
 	if (dev.open(ctx->m_dev))
@@ -511,7 +592,7 @@ int SDPWriteCmd::run(CmdCtx *ctx, void *pbuff, size_t size, uint32_t addr)
 		if (report.write(((uint8_t*)pbuff)+i, sz, 2))
 			return -1;
 
-		if (check_ack(&report, ROM_STATUS_ACK))
+		if (validate && check_ack(&report, ROM_STATUS_ACK))
 			return -1;
 	}
 
